function [pData, pCorrect, nRT, pRT, eRT] = RWcoactive(parms, correctBoundary, nondtdist, rtdata)
% [pDATA, pUPPER, pRT] = RWcoactive(PARMS, NONDTDIST, RTDATA) takes
% inputs PARMS, CORRECTBOUNDARY, NONDTDIST, and RTDATA and returns pDATA, the probability 
% of RT given the parameters in PARMS, pA, the probability of the process 
% ending at the upper boundary, and pRT, the full distribution of RT
% probabilities
%
% PARMS is a vector containing the following parameters in the following
% order: 
%   1) M - Mean drift rate (or ALPHA if driftdist = 'beta')
%   3) BOUNDARIES - Upper and Lower Boundary Locations
%   4) RWSCALE - Scanling parameter to scale random walk to msec (must be > 1)
%   5) RESIDUALS - Log-normal residual mean and variance
% 
% If you want to use the defaults for any of these parameters, just set
% them to be empty vectors (e.g., startdist = [])
%
% RTDATA is a vector of RTDATA in msecs

M = parms(1);                   % Mean drift rate
boundaries = parms(2:3);        % Criterion Upper/Lower
rwScale = parms(4);             % Scaling parameter for steps to msec
residuals = parms(5:6);         % Residual mean and variance

N = 0:3000;

[pCorrect, pN] = rw(M, boundaries, N);

%% Insert scaled steps into larger array (1 msec)
ps = pN(correctBoundary,:); % Take correct boundary only
if correctBoundary == 2; pCorrect = 1 - pCorrect; end

scaledDTsteps = round((1:numel(ps)) * rwScale - rwScale);          % Scale the decision time stemp
dt = [(0:max(scaledDTsteps))', zeros(max(scaledDTsteps) + 1, 1)];  % Make two columns of times and likelihoods
dt(ismember(dt(:,1), scaledDTsteps), 2) = ps;                      % Insert the likelihoods at the appropriate time

%% Convolve decision and nondecision times
switch nondtdist
    case 'uniform'
        ndt = unifpdf(dt(:,1), residuals(1), residuals(2));
    case 'lognormal'
        ndt = (1./dt(:,1)) .* normpdf(log(dt(:,1)), residuals(1), residuals(2)); % Non-decision time is log normal
    case 'normal'
        ndt = normpdf(dt(:,1), residuals(1), residuals(2));
    case 'none'
        ndt = ones(size(dt,1),1);
        pRT = dt(:,2)';
end

if ~strcmp(nondtdist, 'none')
    ndt(1) = 0; % Set first ndt to zero, as log(0) is bad
    pRT = fftConv(dt(:,2), ndt); % Do FFT convolution
end
nRT = ((1:numel(pRT)) - 1);  % Get new time index
pRT(pRT < eps) = 0;
pRT = pRT./pCorrect;
eRT = sum(nRT .* pRT); % Expected RT

%% Compute probability of the data
if exist('rtdata', 'var') == 1
    pData = zeros(numel(rtdata, 1)); % Initialize vector
else
    rtdata = [];
    pData = [];
end
if  ~isempty(rtdata)
    for i = 1:numel(rtdata)
        rtidx = find(rtdata(i) <= nRT, 1, 'first');
        if ~isempty(rtidx)
            pData(i,1) = pRT(rtidx); % Find appropriate probabilities
        else
            pData(i,1) = NaN;
        end
    end
end