% Logical Rule Model - Parallel
function [pData, pCorrect, nRT, pRT, eRT] = RWparallel(parms, correctBoundary, nondtdist, rtdata, selfterm)
% parms = [driftrates, boundaries, residuals, pXfirst, rwScale]
% driftrates - pStep to upper for each process
% boundaries - upper and lower positions (the self-terminating boundary should be the upper boundary)
% rwScale    - scaling factor to change steps to msec

M = parms(1:2);                 % Mean drift rate
boundaries = parms(3:4);        % Criterion Upper/Lower
rwScale = parms(5);             % Scaling parameter for steps to msec
residuals = parms(6:7);         % Residual mean and variance

N = 0:3000;

nProcesses = numel(M); % Currently only set up for 2 processes (X & Y)
if nProcesses ~= 2; error('RWparallel not set up for nProcesses ~= 2'); end


%% Preallocate probabilities for each process
ppA = zeros(1, nProcesses);             % Probability that the process ends at A
ppN  = zeros(2, numel(N)+1, nProcesses); % N boundaries, N steps + 1, N processes

%% Process pidx
% ppA is 1 x nProcesses
% pN  is 2 (Upper, Lower) x N x nProcesses
for pidx = 1:nProcesses
    [ppA(1,pidx), ppN(:,:,pidx)] = rw(M(pidx), boundaries, N);
end

%% Normalize distributions
% Hack to avoid divide by zero
ppA(ppA == 0) = 1e-10;
ppA(ppA == 1) = 1 - 1e-10;

pN(1,:,1) = ppN(1,:,1)./ppA(1);
pN(2,:,1) = ppN(2,:,1)./(1-ppA(1)); % This seems to work too 10:43am (test with diff parms)
pN(1,:,2) = ppN(1,:,2)./ppA(2);
pN(2,:,2) = ppN(2,:,2)./(1-ppA(2)); % This seems to work too 10:43am (test with diff parms)

%% Outcome probabilities - currently only set up for nProcesses = 2
pAA = prod(ppA);              % Probability that all processes end at the upper boundary
pAB = ppA(1) * (1 - ppA(2));  % Probability that process one ends at the upper boundary
pBA = (1 - ppA(1)) * ppA(2);  % Probability that process two ends at the upper boundary
pBB = prod(1 - ppA);          % Probability that both processes end at the lower boundary

%% Minimum distribution for pBB - when the correct boundary is the upper boundary
% When both processes hit the lower boundary the rt is the minimum rt 
mincdf1 = ((1-cumsum(pN(2,:,1))) .* (1-cumsum(pN(2,:,2))));
minpdf1 = abs([0, (diff(mincdf1))]);

% Minimum distribution for pBB - when the correct boundary is the upper boundary
mincdf2 = ((1-cumsum(pN(1,:,1))) .* (1-cumsum(pN(1,:,2)))); % For the second comparison
minpdf2 = abs([0, (diff(mincdf2))]);

%% Maximum distribution for pAA for when the correct boundary is the lower boundary
maxcdf1 = ((cumsum(pN(2,:,1))) .* (cumsum(pN(2,:,2))));
maxpdf1 = abs([0, (diff(maxcdf1))]);

% Maximum distribution for pAA for when the correct boundary is the upper boundary
maxcdf2 = ((cumsum(pN(1,:,1))) .* (cumsum(pN(1,:,2))));
maxpdf2 = abs([0, (diff(maxcdf2))]);

%% Distribution of decision times for parallel model
if selfterm
    if correctBoundary == 2 % Self-termininating item
        ps = pBB * minpdf1 + pBA * pN(2,:,1) + pAB * pN(2,:,2);
        pCorrect = pBB + pBA + pAB;
    else
        ps = pAA * maxpdf2;
        pCorrect = pAA;
    end
else
    if correctBoundary == 1
        ps = maxpdf1;
        pCorrect = pBB + pBA + pAB;
    else
        ps = maxpdf2;
        pCorrect = pAA;
    end
end

%% Insert scaled steps into larger array
if rwScale < 1; rwScale = 1; end
scaledDTsteps = round((1:numel(ps)) * rwScale - rwScale); 
dt = [(0:max(scaledDTsteps))', zeros(max(scaledDTsteps) + 1, 1)];
dt(ismember(dt(:,1), scaledDTsteps), 2) = ps; % The first column is x, the second column is dt

%% Convolve decision and nondecision times
switch nondtdist
    case 'uniform'
        ndt = unifpdf(dt(:,1), residuals(1), residuals(2));
    case 'lognormal'
        ndt = (1./dt(:,1)) .* normpdf(log(dt(:,1)), residuals(1), residuals(2)); % Non-decision time is log normal
    case 'normal'
        ndt = normpdf(dt(:,1), residuals(1), residuals(2));
    case 'none'
        ndt = ones(size(dt,1),1);
        pRT = dt(:,2)';
end

if ~strcmp(nondtdist, 'none')
    ndt(1) = 0; % Set first ndt to zero, as log(0) is bad
    pRT = fftConv(dt(:,2), ndt); % Do FFT convolution
end
nRT = ((1:numel(pRT)) - 1);  % Get new time index
pRT(pRT < eps) = 0;
pRT = pRT./pCorrect;
eRT = sum(nRT .* pRT); % Expected RT
cRT = cumsum(pRT);

%% Compute probability of the data
if exist('rtdata', 'var') == 1
    pData = zeros(numel(rtdata, 1)); % Initialize vector
else
    rtdata = [];
    pData = [];
end
if  ~isempty(rtdata)
    for i = 1:numel(rtdata)
        rtidx = find(rtdata(i) <= nRT, 1, 'first');
        if ~isempty(rtidx)
            pData(1,i) = cRT(rtidx); % Find appropriate probabilities
        else
            pData(1,i) = NaN;
        end
    end
end