function [pA, pN] = rw(varargin)
% [pUPPER, pN] = RW(M, BOUNDS, N) takes inputs M, BOUNDS, N and returns pUPPER, 
% the probability of a random walk terminating at the UPPER boundary, and pN, the 
% probability of a random walk terminating at step N.
%
% M is a scalar, which gives the average probability of taking a
% step towards the UPPER boundary (1 - M is the average probability of 
% taking a step towards the LOWER boundary). 
% 
% BOUNDS, is a vector of UPPER and LOWER boundary positions. NUMEL(BOUNDS) 
% should equal 2. Both BOUNDS should be positive; however, the function 
% will check that BOUNDS(2) - the LOWER bound - is positive and negate it 
% if necessary.  
% 
% N is a vector of step indexes to compute probabilities. 
%
% Default parameters are: M = .5, BOUNDS = [10 10], N = 3000

%% Set up input
numvarargs = length(varargin);
if numvarargs > 7
    error('variableStartDrift_randomwalk:TooManyInputs', ...
        'requires at most 7 optional inputs');
end

% set defaults for optional inputs
optargs = {.5, [10 10], 3000};
% skip any new inputs if they are empty
newVals = cellfun(@(x) ~isempty(x), varargin);
% now put these defaults into the valuesToUse cell array, and overwrite the ones specified in varargin.
optargs(newVals) = varargin(newVals);
% Place optional args in memorable variable names
[M, boundaries, N] = optargs{:};

%%
% Starting location = 0, so center boundary positions
A = boundaries(1);
B = boundaries(2);

A = ceil(A);        % Upper boundary
if sign(B) > 0
    B = -ceil(B);   % Lower boundary - if B is positive, round up to the nearest integer and negate
else
    B = floor(B);   % If B is already negative, round down to the nearest negative integer
end

m = (A - B) - 1;    % Number of transient steps

p = M;
q = 1 - p;
probp = 1;

i = -B; % Index row of starting position

% Variable starting point
Z = zeros(1, m);
Z(i) = 1;
Zidx = find(Z > 0);

%% Compute probability of ending at boundary B and A
pN = 0; pA = 0;
dpA = nan(1, numel(p));
pAA = nan(1, numel(Zidx));

zbounds = [m - Zidx' + 1, Zidx']; % Upper and Lower boundaries for sampling

for idx = 1:numel(Zidx)
    startingsteps(1) = zbounds(idx, 1) - 1;
    startingsteps(2) = zbounds(idx, 2) - 1;
    Nsteps{1} = startingsteps(1):max(N); % Number of steps for which to calculate probabilities
    Nsteps{2} = startingsteps(2):max(N);
    % This should range from the minimum finishing time to maxN
    
    % Probability of ending at the upper boundary
    dpA(p == q) = Zidx(idx)./(m + 1);
    % Probability of ending at the upper boundary
    dpA(p ~= q) = (1 - (q(p ~= q)./p(p ~= q)).^Zidx(idx))./(1 - (q(p ~= q)./p(p ~= q)).^(m+1)); 
    pAA(idx) =  probp * dpA';

    %% Compute probability of ending after N steps
    % Compute first part of the function
    % This gives a matrix of computations that is numel(p) x numel(Nsteps)
    f1_A = repmat(-log(m + 1) + (Nsteps{1} + 1) * log(2), numel(p), 1) +...
        (((Nsteps{1} + 1)/2)' * log(p .* q))' + repmat((((m + 1 - Zidx(idx))/2) * log(p./q))', 1, numel(Nsteps{1}));
    
    f1_B = repmat(-log(m + 1) + (Nsteps{2} + 1) * log(2), numel(p), 1) +...
        (((Nsteps{2} + 1)/2)' * log(p .* q))' + repmat(((Zidx(idx)/2) * log(q./p))', 1, numel(Nsteps{2}));
    
    % Compute the second part of the function
    f2_A = 0; f2_B = 0; % Initialize
    for j = 1:m % Sum over each transient step
        f2_A = f2_A + cos((pi * j)/(m + 1)).^Nsteps{1} * sin((pi * j * (m + 1 - Zidx(idx)))/(m + 1)) * sin((pi * j)/(m + 1));
        f2_B = f2_B + cos((pi * j)/(m + 1)).^Nsteps{2} * sin((pi * j * Zidx(idx))/(m + 1)) * sin((pi * j)/(m + 1));
    end
    
    % Combine parts 1 and 2
    pN_A = exp(f1_A) .* repmat(f2_A, numel(p), 1);
    pN_B = exp(f1_B) .* repmat(f2_B, numel(p), 1);
    
    % Note you need to add one extra zero for the step to the absorbing boundary
    pN_A = [zeros(numel(p), startingsteps(1)+1) pN_A];
    pN_B = [zeros(numel(p), startingsteps(2)+1) pN_B]; % Keep RT distributions at each boundary separated
    zpN(:,:,idx) = [probp * pN_A; probp * pN_B];
    
    % Accumulate for starting point
    pN = pN + Z(Zidx(idx)) * zpN(:,:,idx);
    pA = pA + Z(Zidx(idx)) * pAA(idx);
end
pN(pN < eps) = 0; % If the probability is less than machine precision, set to zero