function [simBoundary, simStep] = simrandomwalk(varargin)
% function [SIMBOUNDARY, SIMSTEP] = simrw(M, S, boundaries, nsamples, driftdist, startdist, startdistparms)
% [SIMBOUNDARY, SIMSTEP] = SIMRW(M, BOUNDS, N, NSAMPLES) 
% takes inputs M, BOUNDS, N and returns NSAMPLES randomly sampled step times 
% and boundary outcomes from a random walk process. 
%
% M is a scalar, which gives the average probability of taking a
% step towards the UPPER boundary (1 - M is the average probability of 
% taking a step towards the LOWER boundary). 
% 
% BOUNDS, is a vector of UPPER and LOWER boundary positions. NUMEL(BOUNDS) 
% should equal 2. Both BOUNDS should be positive; however, the function 
% will check that BOUNDS(2) - the LOWER bound - is positive and negate it 
% if necessary.  
% 
% NSAMPLES are the number of simulated samples to return. 
%
% Default parameters are: M = .5, BOUNDS = [10 10], N = 3000
% 
% SIMBOUNDARY = 1 if process finished at the upper boundary and 0 otherwise
%
% DL - 20/11/11

%% Set up input
numvarargs = length(varargin);
if numvarargs > 7
    error('variableStartDrift_randomwalk:TooManyInputs', ...
        'requires at most 7 optional inputs');
end

% set defaults for optional inputs
optargs = {.5, [10 10], 10000};
% skip any new inputs if they are empty
newVals = cellfun(@(x) ~isempty(x), varargin);
% now put these defaults into the valuesToUse cell array, and overwrite the ones specified in varargin.
optargs(newVals) = varargin(newVals);
% Place optional args in memorable variable names
[M, boundaries, nsamples] = optargs{:};

%%
% Starting location = 0, so center boundary positions
A = boundaries(1);
B = boundaries(2);

A = ceil(A);        % Upper boundary
if sign(B) > 0
    B = -ceil(B);   % Lower boundary - if B is positive, round up to the nearest integer and negate
else
    B = floor(B);   % If B is already negative, round down to the nearest negative integer
end

m = (A - B) - 1;    % Number of transient steps

i = -B;

% Starting point
Z = zeros(1, m);
Z(i) = 1;
Zidx = find(Z > 0);

%% Convert different starting points to different boundary positions
sampBoundaries = [m - Zidx' + 1, Zidx']; % Upper and Lower boundaries for sampling
nBoundSamples  = round(nsamples * Z(Z > 0));

% Correct difference in samples
diffsamps = sum(nBoundSamples) - nsamples;
if diffsamps < 0; % sum(nBoundSamples) < nsamples
    for sidx = 1:abs(diffsamps)
        nidx = find(rand < cumsum(repmat(1/numel(nBoundSamples), 1, numel(nBoundSamples))), 1, 'first');
        nBoundSamples(nidx) = nBoundSamples(nidx) + 1;
    end
elseif diffsamps > 0; % sum(nBoundSamples) > nsamples
    for sidx = 1:abs(diffsamps)
        nidx = find(rand < cumsum(repmat(1/numel(nBoundSamples), 1, numel(nBoundSamples))), 1, 'first');
        nBoundSamples(nidx) = nBoundSamples(nidx) - 1;
    end
end

%% Sample drift rates for each boundary setting
simBoundary = [];
simStep = [];

for idx = 1:numel(nBoundSamples)
    p{idx} = repmat(M, nBoundSamples(idx), 1);
    [finalcnt, finalstep] = simRW(p{idx}, 1, sampBoundaries(idx,:), nBoundSamples(idx));
    simBoundary = [simBoundary; double(finalcnt > 0)];
    simStep     = [simStep; finalstep];
end
randorder = randperm(nsamples);
simStep = simStep(randorder);
simBoundary = simBoundary(randorder);

%% Run N parallel random walks
function [finalcnt, finalstep] = simRW(driftrate, index, boundaries, nsamples)

if boundaries(2) < 0 % Both boudaries should be positive
    boundaries(2) = -boundaries(2);
end
criterion = boundaries; 

rwindex = index; %repmat(index,2,1);

totstep = zeros(size(rwindex,1)  * nsamples, 1);           % Initialize the total step counter
cnt = repmat(0, size(rwindex, 1) * nsamples, 1);  % Initialize the counter starting position
finalcnt = cnt;
finalstep = totstep;
cntrows = (1:(size(rwindex,1) * nsamples))';
rowcp = driftrate(:,1);

while size(cnt, 1) > 0 && any(any([cnt < criterion(1), cnt > -criterion(2)], 2))  % Loop until each counter meets the criterion
    totstep = totstep + 1;                            %   While looping, increment the total step counter

    ev = rand(size(cnt, 1), 1);
    step = double(ev <= rowcp);
    step(step == 0) = -1; 

    cnt = cnt + step;

    if any(any([cnt >= criterion(1), cnt <= -criterion(2)], 2));
        whichrow = find(any([cnt >= criterion(1), cnt <= -criterion(2)], 2));
        finalcnt(cntrows(whichrow),:) = cnt(whichrow,:);
        finalstep(cntrows(whichrow), 1) = totstep(whichrow);

        cnt(whichrow,:) = [];
        cntrows(whichrow) = [];
        rowcp(whichrow,:) = [];
        totstep(whichrow) = [];
    end
end