% Check parallel model predictions against simulation data
% Predictions are exact with no residual time added

clear all
clc
warning('off', 'variableStartDrift_randomwalk:DistParmNotUsed')
rand('state', 23243)

maxXlimit = 3000; % How far out to draw the x-axis in the plot

%% Parameters
xbnd = 1.5775; % Boundary on x
ybnd = 1.6295; % Boundary on y
sx   = 1.5199;  % Standard deviation on x
sy   = 1.85737;  % Standard deviation on y

Upper = 6.253810; % Exhaustive boundary
Lower = 10; % Self-terminating boundary

mu  = 6.8864; % Residual log-normal distribution mean (non-decision time)
smu = 0.44108;  % Residual log-normal distribution std (non-decision time)
k   = 7.1828; % Scaling parm to convert steps to ms
parms = [xbnd, ybnd, sx, sy, Upper, Lower, mu, smu, k];


%% Stimulus coordinates and associated category labels

%  3 |  x1y3 | x2y3    x3y3
%    |       |
%    |       |
%  2 |  x1y2 | x2y2    x3y2
%    |       |---------------
%    |
%  1 |  x1y1 | x2y1    x3y1
%    ------------------------
%         1      2      3

stimuli = [1 1; 1 2; 1 3; 2 1; 2 2; 2 3; 3 1; 3 2; 3 3]; % Mean of perceptual distribution
categories = [2 2 2 2 1 1 2 1 1]'; % 2 = B = Self-terminating category, 1 = A = Exhaustive category


%% Variables
t        = 0:3000; % Number of time steps to measure
nsamples = 100000;  % Number of simulations to run

% Prespecify for speed
cdf = zeros(size(stimuli, 1), numel(t));
pdf = zeros(size(stimuli, 1), numel(t));
output = []; 

%% Compute drift rates from perceptual distributions
p = DBT_driftrate(stimuli, [xbnd ybnd], [sx sy], true); % Drift rates are given as pUpper (probability of stepping toward the upper boundary)
p = 1 - p;
%% Cycle through each stimulus
for i = 1:size(stimuli, 1);
    % Generate simulated values for each dimension
    [simBoundary, simStep] = simrandomwalk(p(i), [Upper Lower], nsamples);
    
    % Convert dimension values to final probabilities
    switch categories(i)
        case 1 % Exhaustive boundary item
            hitA   = simBoundary == 1; % Find simulated walks in which the coactive walk hit the upper boundary
            
            % Preload
            rts      = nan(nsamples, 1);
            
            corrects     = hitA; % If both responses terminate at the exhaustive boundary and the category is 2, the response is correct
            rts(hitA,1) = simStep(hitA, :);
            
        case 2 % Self-terminating boundary item
            hitB   = simBoundary == 0; % Find simulated walks in which the coactive walk hit the upper boundary
            
            % Preload
            corrects = zeros(nsamples, 1);
            rts = nan(nsamples, 1);
            
            % Compute steps
            rts(hitB)  = simStep(hitB, 1);   % The rt
            
            % The response is correct if any of those options occurs
            corrects(hitB)   = 1;
    end
    
    %% Convert step times to msec
    rts(~corrects) = []; % Remove error steps
    simMSEC = rts * k;
    
    %% Add in residual lognormal time
    simRT{i} = simMSEC;% + exp(normrnd(mu, smu, sum(corrects), 1));
    
    [n, e]   = histc(simRT{i}, t); % Count the rts in each time bin
    pdf(i,:) = n/nnz(simRT{i});   % Convert the counts to proportions to find the estimated pdf
    cdf(i,:) = cumsum(n / nnz(simRT{i})); %  Take the cumulative sum to get the cdf
    
    % Return the quantiles, the proportion of correct responses and the mean RT
    output = [output; i, t(find(cdf(i,:) <= .10, 1, 'last')),...
        t(find(cdf(i,:) <= .30, 1, 'last')),...
        t(find(cdf(i,:) <= .50, 1, 'last')),...
        t(find(cdf(i,:) <= .70, 1, 'last')),...
        t(find(cdf(i,:) <= .90, 1, 'last')),...
        sum(corrects)/nsamples, mean(simRT{i})];
    
    %% Model predictions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Allocate the parameters to a vector variable
    parms = [p(i), [Upper Lower], k, mu, smu]; % If category is B
    
    % Use the numerical prediction function to generate predictions
    [pData(i,:), pCorrect(i,:), nRT(i,:), pRT(i,:), eRT(i)] =  RWcoactive(parms, categories(i), 'none', []);
    
    % Return the predicted quantiles, the proportion of correct responses and the mean RT
    pred(i,:) = [nRT(i,find(cumsum(pRT(i,:)) <= .1, 1, 'last')),...
                 nRT(i,find(cumsum(pRT(i,:)) <= .3, 1, 'last')),...
                 nRT(i,find(cumsum(pRT(i,:)) <= .5, 1, 'last')),...
                 nRT(i,find(cumsum(pRT(i,:)) <= .7, 1, 'last')),...
                 nRT(i,find(cumsum(pRT(i,:)) <= .9, 1, 'last')),...
                 pCorrect(i),...
                 eRT(i)];
             
   output = [output; i pred(i,:)];   
end
% % Display the output and the predictions
output

% figure('WindowStyle', 'docked')
% plotLocs = [7 4 1 8 5 2 9 6 3];
% for ii = 1:9
%     subplot(3,3,ii)
%     bar(e(ii,1:20:end), pdf(ii,1:20:end), 'FaceColor', [.73 .73 0])
%     hold on
%     plot(nRT(ii,:), pRT(ii,:), '-r', 'LineWidth', 2)
%     set(gca, 'XLim', [0 maxXlimit])
%     box on
% end